/*

 ============================================================================
                   The Apache Software License, Version 1.1
 ============================================================================

 Copyright (C) 1999-2002 The Apache Software Foundation. All rights reserved.

 Redistribution and use in source and binary forms, with or without modifica-
 tion, are permitted provided that the following conditions are met:

 1. Redistributions of  source code must  retain the above copyright  notice,
    this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.

 3. The end-user documentation included with the redistribution, if any, must
    include  the following  acknowledgment:  "This product includes  software
    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
    Alternately, this  acknowledgment may  appear in the software itself,  if
    and wherever such third-party acknowledgments normally appear.

 4. The names "Apache Cocoon" and  "Apache Software Foundation" must  not  be
    used to  endorse or promote  products derived from  this software without
    prior written permission. For written permission, please contact
    apache@apache.org.

 5. Products  derived from this software may not  be called "Apache", nor may
    "Apache" appear  in their name,  without prior written permission  of the
    Apache Software Foundation.

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
 DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
 OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 This software  consists of voluntary contributions made  by many individuals
 on  behalf of the Apache Software  Foundation and was  originally created by
 Stefano Mazzocchi  <stefano@apache.org>. For more  information on the Apache
 Software Foundation, please see <http://www.apache.org/>.

*/

package org.apache.cocoon.transformation;

import java.io.IOException;

import org.apache.avalon.framework.parameters.Parameters;

import org.apache.cocoon.environment.SourceResolver;

import org.apache.cocoon.transformation.constrained.*;

import java.util.TooManyListenersException;
import java.util.ArrayList;
import java.util.Arrays;

import java.util.Map;

import java.awt.Color;

import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;

import org.krysalis.wings.*;


/**
 * This Transformer transforms XML that describes a chart into an SVG
 * representation.
 * See Cocoon examples for implicit DTD.
 *
 * @author <a href="mailto:nicolaken@apache.org">Nicola Ken Barozzi</a>
 */
public class Chart2SvgTransformer extends AbstractConstrainedTransformer {

    private String        originalPrefix = "unknown";
    private boolean       minSet         = false;
    private boolean       maxSet         = false;
    private ChartModel2D  cm             = new ChartModel2D();
    private String        type           = "";
    private String        title          = "";
    private AbstractChart c;
    private ColorList     cl = new ColorList();
    private double        min;
    private double        max;
    private boolean       LF3D   = false;
    private int           width  = 320;
    private int           height = 200;

    public String getUri() {
        return "http://apache.org/cocoon/chart2svg";
    }

    public String getName() {
        return "chart2svg";
    }

    public void setup(SourceResolver resolver, Map map, String string, Parameters parameters)
    {

    }

    public void init(Parameters parameters) {

            String[] chartdefStack = { "chartdef" };

            addEventListener(new ElementEventAdapter() {

                public void containerElementEnded(ContainerElementEndEvent e)
                        throws SAXException {

                    if (minSet) {
                        cm.setMin(min);
                    }

                    if (maxSet) {
                        cm.setMax(max);
                    }

                    if (type.trim().equals("bar")) {
                        c = new BarChart(cm);

                        ((BarChart) c).set3DLF(LF3D);
                    }
                     else if (type.trim().equals("pie")) {
                        c = new PieChart(cm);
                    }
                     else {
                        c = new LineChart(cm);
                    }

                    c.setTitle(title);
                    c.setColorList(cl);
                    c.setSize(width, height);

                    SVGGraphics svg = new SVGGraphics(Chart2SvgTransformer.super.contentHandler, width,
                                                      height);
                    svg.startEvents();
                    c.paint(svg);
                    svg.stopEvents();


                }
            }, new XmlTreeConstraint(chartdefStack, true));

            String[] typeStack = { "type" };

            addEventListener(new ElementEventAdapter() {

                public void elementValueRecieved(ElementValueEvent e) {
                    type = e.getElementValue();
                }
            }, new XmlTreeConstraint(typeStack, true));

            String[] titleStack = { "title" };

            addEventListener(new ElementEventAdapter() {

                public void elementValueRecieved(ElementValueEvent e) {
                    title = e.getElementValue();
                }
            }, new XmlTreeConstraint(titleStack, true));

            String[] LF3dStack = { "LF3d" };

            addEventListener(new ElementEventAdapter() {

                public void elementValueRecieved(ElementValueEvent e) {
                    LF3D = true;
                }
            }, new XmlTreeConstraint(LF3dStack, true));

            String[] colorStack = { "color" };

            addEventListener(new ElementEventAdapter() {

                public void elementValueRecieved(ElementValueEvent e) {
                    cl.add(Color.decode(e.getElementValue()));
                }
            }, new XmlTreeConstraint(colorStack, true));

            String[] widthStack = { "width" };

            addEventListener(new ElementEventAdapter() {

                public void elementValueRecieved(ElementValueEvent e) {
                    width = Integer.parseInt(e.getElementValue());
                    try
                    {
                    Chart2SvgTransformer.this.start("prova");
                    Chart2SvgTransformer.this.end("prova");
                    Chart2SvgTransformer.this.data("prova data");
                    }
                    catch(Exception exx)
                    {
                      System.out.println("AAAAAAAAAAAAAAAAAAAAAAAAAAA");
                    }
                }
            }, new XmlTreeConstraint(widthStack, true));

            String[] heightStack = { "height" };

            addEventListener(new ElementEventAdapter() {

                public void elementValueRecieved(ElementValueEvent e) {
                    height = Integer.parseInt(e.getElementValue());
                }
            }, new XmlTreeConstraint(heightStack, true));

            String[] minStack = { "min" };

            addEventListener(new ElementEventAdapter() {

                public void elementValueRecieved(ElementValueEvent e) {
                    min    = Double.parseDouble(e.getElementValue().trim());
                    minSet = true;
                }
            }, new XmlTreeConstraint(minStack, true));

            String[] maxStack = { "max" };

            addEventListener(new ElementEventAdapter() {

                public void elementValueRecieved(ElementValueEvent e) {
                    max    = Double.parseDouble(e.getElementValue().trim());
                    maxSet = true;
                }
            }, new XmlTreeConstraint(maxStack, true));

            String[] dataStack = { "data" };

            addEventListener(new DataListener(),
                                         new XmlTreeConstraint(dataStack,
                                             false));
            addEventListener(new RipperListener(),
                                         new XmlTreeConstraint(dataStack,
                                             true));

            String[] appearanceStack = { "appearance" };

            addEventListener(
                new RipperListener(),
                new XmlTreeConstraint(appearanceStack, true));

            String[] propertiesStack = { "properties" };

            addEventListener(
                new RipperListener(),
                new XmlTreeConstraint(propertiesStack, true));

            String[] colorsStack = { "colors" };

            addEventListener(new RipperListener(),
                                         new XmlTreeConstraint(colorsStack,
                                             true));

    }


    class DataListener extends ElementEventAdapter {

        public void elementValueRecieved(ElementValueEvent e)
                throws SAXException {
            cm.addToRow(e.getElementName(),
                        Double.parseDouble(e.getElementValue()));
        }
    }

}
