/*
 * $Header: /home/cvspublic/xml-cocoon2/src/scratchpad/src/org/apache/cocoon/validation/schematron/Attic/SchematronFactory.java,v 1.1.2.3 2002/11/21 15:06:52 sylvain Exp $
 * $Revision: 1.1.2.3 $
 * $Date: 2002/11/21 15:06:52 $
 *
 * ====================================================================
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999-2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Commons", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 2001, Plotnix, Inc,
 * <http://www.plotnix.com/>.
 * For more information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

package org.apache.cocoon.validation.schematron;

import java.io.OutputStreamWriter;
import java.io.InputStream;

// java classes
import java.util.Properties;
import java.util.List;
import java.io.IOException;
import java.net.URL;
 
// XML classes
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import org.w3c.dom.Node;
import org.w3c.dom.Element;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;

// logger
import org.apache.log.Hierarchy;
import org.apache.log.Logger;
import org.apache.log.Priority;

// JXPath classes
import org.apache.commons.jxpath.JXPathContext;

// Cocoon classes
import org.apache.avalon.framework.CascadingRuntimeException;
import org.apache.cocoon.validation.SchemaFactory;
import org.apache.cocoon.validation.Schema;


/**
 * A helper class which builds a SchematronSchema instance object
 * from a DOM source
 *
 * @author Ivelin Ivanov, ivelin@acm.org, ivelin@iname.com
 */
public class SchematronFactory extends SchemaFactory
{
  
  /**
   * the schema name space prefix used in the schema document
   */
  private String schemaPrefix_;
  
  /**
   * the default schema name space prefix 
   */
  private String defaultSchemaPrefix_ = "sch";

  /*
   * private logger
   */
  private Logger logger = setupLogger();
  
  
  //
  // Constructors
  //

  /**
   * initialize logger
   */
  protected Logger setupLogger()
  {
    Logger logger = Hierarchy.getDefaultHierarchy().getLoggerFor("XmlForm");
    logger.setPriority( Priority.ERROR );
    return logger;
  }
  
  /**
   * Builds a new Schema instance from 
   * the given XML InputSource
   *
   * @param schemaSrc
   *        the Schema document XML InputSource
   */
  public Schema compileSchema(InputSource schemaSrc)
                              throws InstantiationException
  {
    SchematronSchema schema = null;
    try {

      // load Schema file into a DOM document
      DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance ();
      DocumentBuilder dbld = dbf.newDocumentBuilder ();
      Document document = dbld.parse( schemaSrc );          

      schema = buildSchema( document );

    } catch (Exception e) {
        logger.error("!!! Failed loading Schematron schema", e);
        throw new CascadingRuntimeException(" !!! Failed loading Schematron schema", e);
    }
   return schema;
  } // build


  /**
   * Build Schematron schema object from a DOM document
   * @ param doc DOM document containing the schema
   *
   */
  protected SchematronSchema buildSchema( Document doc )
  {
    SchematronSchema schema = new SchematronSchema();
    boolean errors = false;

    doc.getNamespaceURI ();
    doc.getPrefix ();

    // Initialize the JXPath context
    Element root = doc.createElement ( "root" );
    Element schemaElement = doc.getDocumentElement ();
    schemaPrefix_ = schemaElement.getPrefix ();
    root.appendChild (  schemaElement );
    JXPathContext jxpContext = JXPathContext.newContext ( root );
    jxpContext.setLenient(false);

    // Bind sch:schema element
    
    // schema title
    String title = (String) jxpContext.getValue ( "/schema/title", String.class );
    schema.setTitle( title );
    logger.debug( "Schema title: " + schema.getTitle());
    
    bindPatterns( schema, jxpContext );
    bindPhases( schema, jxpContext );
   
    return schema;
  }
  
  /**
   * populates the patterns elements from the dom tree
   *
   * @param schema the schema instance
   * @param jxpContext 
   */
  protected void bindPatterns( SchematronSchema schema, JXPathContext jxpContext)
  {
    // ensure that mandatory elements which are not found
    // will result in Exception
    jxpContext.setLenient(false);

    // schema patterns
    int ptCount = ((Integer) jxpContext.getValue ( "count(/schema/pattern)", Integer.class )).intValue();
    logger.debug( "\nNumber of patterns:  " + ptCount);
    for (int i = 1; i <= ptCount; i++)
    {
      logger.debug( "Pattern# :  " + i);
      Pattern pattern = new Pattern();
      String ptprefix = "/schema/pattern[" + i + "]";

      String name = (String) jxpContext.getValue ( ptprefix + "/@name", String.class ); 
      pattern.setName( name );
      logger.debug( "Pattern name :  " + pattern.getName());

      String id = (String) jxpContext.getValue (  ptprefix + "/@id", String.class ); 
      pattern.setId( id );
      logger.debug( "Pattern id :  " + pattern.getId() );
      
      bindRules( pattern, ptprefix, jxpContext );
      
      schema.addPattern( pattern );
    }
  }

  
  /**
   * populates the rules elements for a pattern
   * from the dom tree
   *
   * @param pattern 
   * @param pathPrefix pattern path prefix
   * @param jxpContext JXPathContext
   */
  protected void bindRules( Pattern pattern, String pathPrefix, JXPathContext jxpContext)
  {
    // ensure that mandatory elements which are not found
    // will result in Exception
    jxpContext.setLenient(false);

    // schema rules
    int ruleCount = ((Integer) jxpContext.getValue ( "count(" + pathPrefix + "/rule)", Integer.class )).intValue();
    logger.debug( "\nNumber of rules:  " + ruleCount);
    for (int i = 1; i <= ruleCount; i++)
    {
      logger.debug( "Rule# :  " + i);
      Rule rule = new Rule();
      String rulePrefix = pathPrefix + "/rule[" + i + "]";

      String context = (String) jxpContext.getValue ( rulePrefix + "/@context", String.class ); 
      rule.setContext( context );
      logger.debug( "Rule context :  " + rule.getContext());

      bindAsserts( rule, rulePrefix, jxpContext );
      
      pattern.addRule( rule );
    }
  }

  
  /**
   * populates the assert elements for a rule
   * from the dom tree
   *
   * @param rule 
   * @param pathPrefix rule path prefix
   * @param jxpContext JXPathContext
   */
  protected void bindAsserts( Rule rule, String pathPrefix, JXPathContext jxpContext)
  {
    // ensure that mandatory elements which are not found
    // will result in Exception
    jxpContext.setLenient(false);

    // schema reports
    int elementCount = ((Integer) jxpContext.getValue ( "count(" + pathPrefix + "/assert)", Integer.class )).intValue();
    logger.debug( "\nNumber of asserts:  " + elementCount);
    for (int i = 1; i <= elementCount; i++)
    {
      logger.debug( "Assert# :  " + i);
      Assert anAssert = new Assert();
      String assertPrefix = pathPrefix + "/assert[" + i + "]";

      String test = (String) jxpContext.getValue ( assertPrefix + "/@test", String.class ); 
      anAssert.setTest( test );
      logger.debug( "Assert test :  " + anAssert.getTest());


      // since diagnostics is a non-mandatory element
      // we will try to get its value in a lenient mode
      jxpContext.setLenient(true);
      String diagnostics = (String) jxpContext.getValue ( assertPrefix + "/@diagnostics", String.class ); 
      anAssert.setDiagnostics( diagnostics );
      logger.debug( "Assert diagnostics :  " + anAssert.getDiagnostics());
      jxpContext.setLenient(false);

      String message = (String) jxpContext.getValue ( assertPrefix, String.class ); 
      anAssert.setMessage( message );
      logger.debug( "Assert message :  " + anAssert.getMessage());

      rule.addAssert( anAssert );
    }
  }
  
  
  /**
   * populates the assert elements for a rule
   * from the dom tree
   *
   * @param rule 
   * @param pathPrefix rule path prefix
   * @param jxpContext JXPathContext
   */

  protected void bindRerports( Rule rule, String pathPrefix, JXPathContext jxpContext)
  {
    // ensure that mandatory elements which are not found
    // will result in Exception
    jxpContext.setLenient(false);

    // schema reports
    int elementCount = ((Integer) jxpContext.getValue ( "count(" + pathPrefix + "/report)", Integer.class )).intValue();
    logger.debug( "\nNumber of reports:  " + elementCount);
    for (int i = 1; i <= elementCount; i++)
    {
      logger.debug( "Report# :  " + i);
      Report report = new Report();
      String assertPrefix = pathPrefix + "/report[" + i + "]";

      String test = (String) jxpContext.getValue ( assertPrefix + "/@test", String.class ); 
      report.setTest( test );
      logger.debug( "Report test :  " + report.getTest());

      // since diagnostics is a non-mandatory element
      // we will try to get its value in a lenient mode
      jxpContext.setLenient(true);
      String diagnostics = (String) jxpContext.getValue ( assertPrefix + "/@diagnostics", String.class ); 
      report.setDiagnostics( diagnostics );
      logger.debug( "Report diagnostics :  " + report.getDiagnostics());
      jxpContext.setLenient(false);


      String message = (String) jxpContext.getValue ( assertPrefix, String.class ); 
      report.setMessage( message );
      logger.debug( "Report message :  " + report.getMessage());

      rule.addReport( report );
    }
  }

  
  /**
   * populates the phases elements from the dom tree
   *
   * @param schema the schema instance
   * @param jxpContext 
   */
  protected void bindPhases( SchematronSchema schema, JXPathContext jxpContext)
  {
    // ensure that mandatory elements which are not found
    // will result in Exception
    jxpContext.setLenient(false);

    // schema phases
    int phaseCount = ((Integer) jxpContext.getValue ( "count(/schema/phase)", Integer.class )).intValue();
    logger.debug( "\nNumber of phases:  " + phaseCount);
    
    for (int i = 1; i <= phaseCount; i++)
    {
      logger.debug( "phase# :  " + i);
      Phase phase = new Phase();
      String phprefix = "/schema/phase[" + i + "]";

      String id = (String) jxpContext.getValue ( phprefix + "/@id", String.class ); 
      phase.setId( id );
      logger.debug( "phase id :  " + phase.getId());

      bindPhaseActivePatterns( phase, phprefix, jxpContext );
      
      schema.addPhase( phase );
    }
  }
  
  
  protected void bindPhaseActivePatterns( Phase phase, String pathPrefix, JXPathContext jxpContext)
  {
    // ensure that mandatory elements which are not found
    // will result in Exception
    jxpContext.setLenient(false);

    // phase active patterns
    int elementCount = ((Integer) jxpContext.getValue ( "count(" + pathPrefix + "/active)", Integer.class )).intValue();
    logger.debug( "Number of active patterns:  " + elementCount);
    for (int i = 1; i <= elementCount; i++)
    {
      logger.debug( "active pattern # :  " + i);
      ActivePattern activePattern = new ActivePattern();
      String assertPrefix = pathPrefix + "/active[" + i + "]";

      String pt = (String) jxpContext.getValue ( assertPrefix + "/@pattern", String.class ); 
      activePattern.setPattern( pt );
      logger.debug( "Phase active pattern :  " + activePattern.getPattern());

      phase.addActive( activePattern  );
    }
  }

  
  
  
  /*
   * Replace all occurances of sch: with the actual Schema prefix used in the document
   *
   * @todo fix this implementaion. There are problems with DOM. 
   * Returns null instead of the actual namespace prefix (e.g. "sch") as expected.  
   */
  protected String fixns( String path )
  {
    // Ironicly, at the time I am writing this
    // JDK 1.4 is offering String.replaceAll(regex, str)
    // I don't use it however for backward compatibility
    StringBuffer strbuf = new StringBuffer( path );
    int i = 0;
    int j = 0;
    String dprefix = defaultSchemaPrefix_ + ":";
    int dplen = dprefix.length();
    while ( ( j = path.indexOf ( dprefix, i ) ) >= 0 )
    {
      strbuf.append ( path.substring ( i, j ) );
      strbuf.append ( schemaPrefix_ );
      strbuf.append ( ':' );
      i = j + dplen;
    }
    strbuf.append( path.substring ( i ) );
    return strbuf.toString ();
  }


}
