/*
 * $Header: /home/cvspublic/xml-cocoon2/src/scratchpad/src/org/apache/cocoon/xmlform/Attic/FormBeanBinder.java,v 1.1.2.2 2002/11/20 12:35:18 cziegeler Exp $
 * $Revision: 1.1.2.2 $
 * $Date: 2002/11/20 12:35:18 $
 *
 * ====================================================================
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999-2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Commons", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 2001, Plotnix, Inc,
 * <http://www.plotnix.com/>.
 * For more information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

package org.apache.cocoon.xmlform;

import java.util.Enumeration;
import java.util.Collection;

import org.apache.cocoon.environment.Request;
import org.apache.commons.jxpath.JXPathContext;
import org.apache.commons.jxpath.Pointer;

/**
 * <p>
 *  Binds an HTML Form to a JavaBean or DOM node.
 *  Expects that all request parameter names are XPath expressions to properties of the bean.
 *  For each request parameter, finds and assigns its value to the
 *  JavaBean property corresponding to the parameter's name
 *  @todo provide a simple example
 * </p>
 *
 *
 * @author Ivelin Ivanov
 * @version $Revision: 1.1.2.2 $ $Date: 2002/11/20 12:35:18 $
 */

public class FormBeanBinder  
{

    /**
   * bind request parameters to JavaBean properties
   * or DOM nodes
   * @param request the submitetd HttpServletRequest
   * @param jBean the bean which this method populates from the request
   */
    public static void bind(Request request, Object jBean)
      {
      JXPathContext context = JXPathContext.newContext(jBean);
      context.setLenient( false );
      Enumeration enum = request.getParameterNames ();
      while (enum.hasMoreElements ())
        {
          String path = (String) enum.nextElement ();
          Object[] values = request.getParameterValues ( path );

          Pointer pointer = context.getPointer( path );
          Object property = pointer.getValue();
          // if the property is a collection, set value as array
          if ( property instanceof Collection )
          {
            pointer.setValue( values );
          }
          // otherwise set the value of the first element (there shouldn't be other)
          // in the values array
          else
          {
            pointer.setValue( values[0] );
          }
       } // while
    }

}

